using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Mail;
using System.Text;

namespace Kruskal
{
    class Program
    {
        private static int[,] Routes = null;
        private static SortedSet<Edge> Edges = new SortedSet<Edge>();
        private static Dictionary<int, int> utils = new Dictionary<int, int>();

        static int Main()
        {
            int numberOfTowns = 0;
            int numberOfRoads = 0;

            ReadFromFile(ref numberOfRoads, ref numberOfTowns);

            var tree = new int[numberOfRoads + 1];
            var minSpanTree = new HashSet<Edge>();
            int treesCount = 1;

            treesCount = FindMinimumSpanningTree(tree, minSpanTree, treesCount);

            PrintMinimumSpanningTree(minSpanTree);

            return 0;
        }

        private static int FindMinimumSpanningTree(int[] tree, HashSet<Edge> minSpanTree, int treesCount)
        {
            foreach (var edge in Edges)
            {
                if (tree[edge.StartNode] == 0) // not visited
                {
                    if (tree[edge.EndNode] == 0) // both ends are not visited
                    {
                        tree[edge.StartNode] = tree[edge.EndNode] = treesCount;
                        treesCount++;
                    }
                    else
                    {
                        // attach the start node to the tree of the end node
                        tree[edge.StartNode] = tree[edge.EndNode];
                    }

                    minSpanTree.Add(edge);
                }
                else // the start is part of a tree
                {
                    if (tree[edge.EndNode] == 0)
                    {
                        //attach the end node to the tree;
                        tree[edge.EndNode] = tree[edge.StartNode];
                        minSpanTree.Add(edge);
                    }
                    else if (tree[edge.EndNode] != tree[edge.StartNode]) // combine the trees
                    {
                        int oldTreeNumber = tree[edge.EndNode];

                        for (int i = 0; i < tree.Length; i++)
                        {
                            if (tree[i] == oldTreeNumber)
                            {
                                tree[i] = tree[edge.StartNode];
                            }
                        }
                        minSpanTree.Add(edge);
                    }
                }
            }
            return treesCount;
        }

        private static void ReadFromFile(ref int numberOfRoads, ref int numberOfTowns)
        {
            using (var reader = new StreamReader("graph.in"))
            {
                string line = reader.ReadLine();
                int counter = 0;
                while (line != null)
                {
                    counter++;
                    string[] curLine = line.Split(' ');
                    if (counter == 1)
                    {
                        numberOfTowns = int.Parse(curLine[0]);
                        numberOfRoads = int.Parse(curLine[1]);
                        Routes = new int[numberOfTowns, numberOfTowns];
                    }
                    else if (counter <= numberOfRoads + 1)
                    {
                        Edges.Add(new Edge(int.Parse(curLine[0]), int.Parse(curLine[1]), int.Parse(curLine[2]), counter - 1));
                    }
                    else
                    {
                        var row = line.Split(' ');
                        for (int i = 0; i < numberOfTowns; i++)
                        {
                            int routeRow = counter - (numberOfRoads + 2);
                            int routeCol = i;
                            int routesCount = int.Parse(row[i]);
                            Routes[routeRow, routeCol] = routesCount;
                        }
                    }

                    line = reader.ReadLine();
                }
            }
        }
  
        private static void PrintMinimumSpanningTree(HashSet<Edge> usedEdges)
        {
            StringBuilder sb = new StringBuilder();
            foreach (var edge in usedEdges)
            {
                sb.AppendLine(edge.Index.ToString());
            }

            var writer = new StreamWriter("graph.out");
            using (writer)
            {
                writer.Write(sb);
            }
        }
    }
}